#pragma once
#include "wmpacket.h"
#include "mesh.h"

/** Allows for mesh data traversal in the World Machine device network.
	@ingroup data
*/
class MeshPacket :
	public WMAreaPacket
{
public:
	static MeshPointer CreateNew();
	MeshPacket &operator=(const MeshPacket &other);

	virtual int getType() { return WMP_mesh; };
	virtual char *getTypeName() { return "Mesh"; };
	virtual bool queryType(const int type);	
	virtual dword Color() { return 0x00ff0000; };

	virtual size_t getHeapUsage();

	virtual PacketPointer Clone() const;			// Return a copy of this WMPacket-derived packet

	virtual bool isTypePageable() { return false; };
	/// TODO *** MeshPacket does not currently support Load operation
	virtual bool Load(std::istream &input) { assert(false); return false; };	
	/// TODO *** MeshPacket does not currently support Save operation
	virtual bool Save(std::ostream &output) { assert(false); return false; };

	/// Retrieve internal TerrainMesh object. May be mutated.
	TerrainMesh &get() { return mesh; };

	virtual void Clear();

private:
	friend class WMPacketFactory;
	MeshPacket(void);
	virtual ~MeshPacket(void);

	virtual void resetContents() { };

	TerrainMesh	mesh;
};

/** Safe Accessor inline function to provide a safe way to cast WMPacket to MeshPacket.
 in debug mode it will assert if the packet is not a MeshPacket. In all modes, it will return
 NULL if incompatible types.
@relates MeshPacket
 */
inline MeshPointer MESH(PacketPointer pack) { 
	if (pack) {
		bool ok = pack->getType() == WMP_mesh;
		if (!ok)
			return NULL;
	}
	return MeshPointer((MeshPacket*) pack.get()); 
};

inline MeshPointer MeshPacket::CreateNew() {
	return MESH(WorldMachine::core->memory->GetPacket(WMP_mesh));
};
