#pragma once

namespace WorldMachine {
/**
 	A simple class to automate keeping track of progress towards a goal. The idea is that
	during a long sequence of computations, we want to "checkin" a certain number of times with the parent.

	This is used mostly to help devices easily decide when to report out to the core that they are making progress.

	Construct with the total "size" of some task and also the number of times you would like to report.

	Then call increment() total_size times during processing. Increment will return true when it is time to check in;
	this will happen number_checkins number of times evenly during the work.

 */
class ProgressAccumulator
{
public:
	ProgressAccumulator(size_t total_size, int number_checkins);

	/** Call once for each step you make towards your goal.
	 ** Will return true after certain steps such that the number of times true
	 ** is the same as requested_steps after being incremented number_checkins times.
	*/
	bool increment();

	/// the number of checkins that have been made so far.
	int getTotal() { return total; };
private:
	int steps_this_cycle, stepsize;
	int total, total_max;
};
}